/*
    Copyright (C) 2011 Harald Sitter <sitter@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Media.h"

#include <QtCore/QDebug>

#include <vlc/vlc.h>

#include "LibVLC.h"

namespace CuteVlc {

//////////////////////////////////// Private ///////////////////////////////////

class MediaPrivate
{
public:
    MediaPrivate(Media *q) :
        q_ptr(q),
        parsed(false),
        duration(-1),
        media(0),
        state(libvlc_NothingSpecial)
    {
    }

    ~MediaPrivate()
    {
        libvlc_event_manager_t *eventManager = libvlc_media_event_manager(media);
        libvlc_event_detach(eventManager, libvlc_MediaMetaChanged, mediaMetaChanged, this);
        libvlc_event_detach(eventManager, libvlc_MediaDurationChanged, durationChanged, this);
        libvlc_event_detach(eventManager, libvlc_MediaStateChanged, mediaStateChanged, this);
        libvlc_event_detach(eventManager, libvlc_MediaSubItemAdded, subItemAdded, this);
        libvlc_event_detach(eventManager, libvlc_MediaParsedChanged, parseChanged, this);
        libvlc_media_release(media);
    }

    void init()
    {
        libvlc_event_manager_t *eventManager = libvlc_media_event_manager(media);
        libvlc_event_attach(eventManager, libvlc_MediaMetaChanged, mediaMetaChanged, this);
        libvlc_event_attach(eventManager, libvlc_MediaDurationChanged, durationChanged, this);
        libvlc_event_attach(eventManager, libvlc_MediaStateChanged, mediaStateChanged, this);
        libvlc_event_attach(eventManager, libvlc_MediaSubItemAdded, subItemAdded, this);
        libvlc_event_attach(eventManager, libvlc_MediaParsedChanged, parseChanged, this);

        parsed = libvlc_media_is_parsed(media);
        state = libvlc_media_get_state(media);

#warning FIXME: subitems not taken into account
    }

    void _q_fetchMetaInformation(int);

#define CUTEVLC_DECLARE_EVENT_CB(x) static void x(const libvlc_event_t *, void *)
    CUTEVLC_DECLARE_EVENT_CB(mediaMetaChanged);
    CUTEVLC_DECLARE_EVENT_CB(durationChanged);
    CUTEVLC_DECLARE_EVENT_CB(mediaStateChanged);
    CUTEVLC_DECLARE_EVENT_CB(subItemAdded);
    CUTEVLC_DECLARE_EVENT_CB(parseChanged);
#undef CUTEVLC_DECLARE_EVENT_CB

    // Workaround for stupid preprocessor not eating templates. Used for metaDataChanged.
    typedef QMap<libvlc_meta_t, QString> MetaDataMap;

#define CUTEVLC_DECLARE_SIGNAL_EMISSION(x,y) void _q_##x(y arg) { V_Q(Media); emit q->x(arg); }
    CUTEVLC_DECLARE_SIGNAL_EMISSION(durationChanged, qint64)
    CUTEVLC_DECLARE_SIGNAL_EMISSION(stateChanged, libvlc_state_t)
    CUTEVLC_DECLARE_SIGNAL_EMISSION(metaDataChanged, MetaDataMap)
#undef CUTEVLC_DECLARE_SIGNAL_EMISSION

#define CUTEVLC_DECLARE_SIGNAL_EMISSION(x) void _q_##x() { V_Q(Media); emit q->x(); }
    CUTEVLC_DECLARE_SIGNAL_EMISSION(parsingFinished)
#undef CUTEVLC_DECLARE_SIGNAL_EMISSION

    Media *const q_ptr;

    bool parsed;

    qint64 duration;
    QMap<libvlc_meta_t, QString> metaData;

    libvlc_media_t *media;
    libvlc_state_t state;

private:
    V_DECLARE_PUBLIC(Media)
};

void MediaPrivate::_q_fetchMetaInformation(int metaType)
{
    V_Q(Media);
    libvlc_meta_t vlcMetaType = static_cast<libvlc_meta_t>(metaType);
    // TODO is fromutf8 appropriate?
    QString value = QString::fromUtf8(libvlc_media_get_meta(media, vlcMetaType));
    qDebug() << Q_FUNC_INFO << "metatype: " << vlcMetaType << " value: " << value;
    metaData[vlcMetaType] = value;

    // FIXME introduce emission function to ensure it is queued and does not block vlc...
    emit q->metaDataChanged(metaData);
}

void MediaPrivate::mediaMetaChanged(const libvlc_event_t *event, void *vlcThat)
{
    Q_ASSERT(vlcThat);
    MediaPrivate *that = static_cast<MediaPrivate *>(vlcThat);

    if (event->u.media_meta_changed.meta_type == libvlc_meta_Publisher ||
        event->u.media_meta_changed.meta_type == libvlc_meta_NowPlaying)
    {
        /* Skip those meta. We don't really care about them for now.
         * And they occure a lot */
        return;
    }

    // FIXME queue, somehow this blocks to death though?!!!!
    QMetaObject::invokeMethod(that->q_func(),
                              "_q_fetchMetaInformation",
                              Qt::DirectConnection,
                              Q_ARG(int, static_cast<int>(event->u.media_meta_changed.meta_type)));
}

void MediaPrivate::durationChanged(const libvlc_event_t *event, void *vlcThat)
{
    Q_ASSERT(vlcThat);
    MediaPrivate *that = static_cast<MediaPrivate *>(vlcThat);

    that->duration = event->u.media_duration_changed.new_duration;

    // FIXME introduce emission function to ensure it is queued and does not block vlc...
    emit that->q_ptr->durationChanged(that->duration);
}

void MediaPrivate::mediaStateChanged(const libvlc_event_t *event, void *vlcThat)
{
    Q_ASSERT(vlcThat);
    MediaPrivate *that = static_cast<MediaPrivate *>(vlcThat);

    libvlc_state_t newState = event->u.media_state_changed.new_state;
    that->state = newState;

    // FIXME introduce emission function to ensure it is queued and does not block vlc...
    emit that->q_ptr->stateChanged(newState);
}

void MediaPrivate::subItemAdded(const libvlc_event_t *event, void *vlcThat)
{
    Q_ASSERT(vlcThat);
    MediaPrivate *that = static_cast<MediaPrivate *>(vlcThat);

    libvlc_media_list_t *subItems = libvlc_media_subitems(that->media);
    Q_ASSERT_X(subItems, "subItems null", "the subitems list should not be null when the subItemAdded callback was received");


    // TODO actually build a list....
    // probably needs a medialist typedef qlist<media> or somesuch
}

void MediaPrivate::parseChanged(const libvlc_event_t *event, void *vlcThat)
{
    Q_ASSERT(vlcThat);
    MediaPrivate *that = static_cast<MediaPrivate *>(vlcThat);

    that->parsed = static_cast<bool>(event->u.media_parsed_changed.new_status);

    qDebug() << Q_FUNC_INFO << that->parsed;

    if (!that->parsed)
        return;

    // FIXME queue??!?!
    QMetaObject::invokeMethod(that->q_func(),
                              "_q_parsingFinished",
                              Qt::DirectConnection);
}

//////////////////////////////////// Public ////////////////////////////////////

Media::Media(const QString &mrl, QObject *parent) :
    QObject(parent),
    d_ptr(new MediaPrivate(this))
{
    V_D(Media);
    d->media = libvlc_media_new_location(libvlc, mrl.toLatin1().constData());
    d->init();
}

Media::Media(libvlc_media_t *media, QObject *parent) :
    QObject(parent),
    d_ptr(new MediaPrivate(this))
{
    V_D(Media);
    libvlc_media_retain(media);
    d->media = media;
    d->init();
}

Media::~Media()
{
    delete d_ptr;
}

libvlc_state_t Media::state() const
{
    V_D(const Media);
    return d->state;
}

libvlc_media_t *Media::vlcInstance() const
{
    V_D(const Media);
    return d->media;
}

bool Media::isParsed() const
{
    V_D(const Media);
    return d->parsed;
}

void Media::addOption(const QString &option)
{
    V_D(Media);
    libvlc_media_add_option(d->media, option.toUtf8().data());
}

void Media::addOptionFlag(const QString &option, unsigned int flags)
{
    V_D(Media);
    libvlc_media_add_option_flag(d->media, option.toUtf8().constData(), flags);
}

void Media::addOptionFlag(const QString &option, void *functionPtr, unsigned int flags)
{
    V_D(Media);
    QString optionWithPtr = option;
    if (optionWithPtr.contains(QLatin1String("%1"))) {
        optionWithPtr.arg(QString::number(static_cast<qint64>(reinterpret_cast<intptr_t>(functionPtr))));
    } else {
        optionWithPtr.append(QString::number(static_cast<qint64>(reinterpret_cast<intptr_t>(functionPtr))));
    }
    libvlc_media_add_option_flag(d->media, optionWithPtr.toUtf8().constData(), flags);
}

void Media::parse()
{
    V_D(Media);
    libvlc_media_parse_async(d->media);
}

} // namespace QtVlc

#include "moc_Media.cpp"
