/*
    Copyright (C) 2011 Harald Sitter <sitter@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CUTEVLC_MEDIA_H
#define CUTEVLC_MEDIA_H

#include <QtCore/QObject>

#include <QtCore/QMap>

extern "C"
{
#include <vlc/libvlc.h>
#include <vlc/libvlc_media.h>
}

#include "Global.h"

class QString;

namespace CuteVlc {

class MediaPrivate;

/**
 * \brief A tiny Qt wrapper around LibVLC's media
 *
 * This class is neither thread-safe nor reentrant.
 *
 * The thread/eventloop in which the Media lives must not be blocked, especially
 * not if you are blocking because you are waiting for a signal from the Media
 * object. Most callbacks from LibVLC get translated into a Qt method invocation
 * to return the callback as soon as possible. If however the eventloop is blocked
 * those invocations will never get executed and you find yourself in a deadlock.
 * Ideally you move the Media to an own thread right away to avoid this.
 *
 * \author Harald Sitter <sitter@kde.org>
 */
class CUTEVLC_EXPORT Media : public QObject
{
    Q_OBJECT
public:
    /**
     * Constructor.
     *
     * Constructs a new Media from a given media location. In particular this will
     * use libvlc's libvlc_media_new_location.
     *
     * \param mrl the media location from which the Media object is to be created.
     */
    explicit Media(const QString &mrl, QObject *parent = 0);

    /**
     * Constructor.
     *
     * Constructs a new Media from a given libVLC media instance.
     *
     * \param media the libVLC media instance.
     */
    explicit Media(libvlc_media_t *media, QObject *parent = 0);

    /**
     * Destructor.
     *
     * Please note that the underlying libvlc media will be released upon destruction.
     */
    virtual ~Media();

    /**
     * \returns the state this Media is currently in.
     *
     * \see stateChanged
     */
    // FIXME maybe ptr to get rid of include?
    libvlc_state_t state() const;

    /**
     * \returns the actual libvlc media instance held by this Media object.
     */
    libvlc_media_t *vlcInstance() const;

    /**
     * \returns whether this Media is already parsed
     */
    bool isParsed() const;

    /**
     * Add an option to the Media.
     *
     * This option will be used to determine how the MediaPlayer will
     * read the Media. This allows to use VLC's advanced
     * reading/streaming options on a per-media basis.
     *
     * The options are detailed in vlc --long-help, for instance "--sout-all"
     *
     * \param option the option to add
     *
     * \see addOptionFlag
     */
    void addOption(const QString &option);

    /**
     * Add an option to the media with configurable flags.
     *
     * This option will be used to determine how the media_player will
     * read the media. This allows to use VLC's advanced
     * reading/streaming options on a per-media basis.
     *
     * The options are detailed in vlc --long-help, for instance "--sout-all"
     *
     * \param option the option to add
     * \param flags the flags for this option
     *
     * \see addOption
     */
    void addOptionFlag(const QString &option, unsigned int flags = libvlc_media_option_trusted);

    /**
     * Add an option to the media with a function pointer and configurable flags.
     *
     * This is a convenience overload for addOptionFlag to append the address
     * of a function pointer to the option. The function pointer must point to a
     * public static function that matches the expected signature.
     *
     * Alternatively if the option string contains "%1" rather than appending, the
     * function pointer address will be used to replace the %1 using QString::arg.
     *
     * \code
     * Media m("/home/user/file.webm");
     * m.addOptionFlag(QString(":sout-smem-audio-prerender-callback="), &lockCall);
     * \endcode
     *
     * \param option the option to add
     * \param functionPtr a function ptr to append to option
     * \param flags the flags for this option
     *
     * \see addOptionFlag
     */
    void addOptionFlag(const QString &option, void *functionPtr,
                       unsigned int flags = libvlc_media_option_trusted);

signals:
    /**
     * Emitted when the duration of the Media changed, e.g. when the duration
     * actually becomes known.
     *
     * \param newDuration the new duration
     */
    void durationChanged(qint64 newDuration);

    /**
     * Emitted when the state of the Media changed.
     *
     * \param newState the new state
     *
     * \see state
     */
    void stateChanged(libvlc_state_t newState);

    /**
     * Emitted when the meta data map of the Media changed.
     *
     * \param metaData the current meta data map
     */
    void metaDataChanged(QMap<libvlc_meta_t, QString> metaData);

    /**
     * Emitted when the Media parsing is finished.
     */
    void parsingFinished();

public slots:
    /**
     * Parse a Media.
     *
     * This fetches (local) meta data and tracks information.
     * Please note that the actual information is emitted using signals.
     *
     * \see durationChanged
     * \see stateChanged
     * \see metaDataChanged
     * \see parsingFinished
     */
    void parse();

private:
    MediaPrivate *const d_ptr;

    V_DISABLE_COPY(Media)
    V_DECLARE_PRIVATE(Media)

    Q_PRIVATE_SLOT(d_func(), void _q_fetchMetaInformation(int))
    Q_PRIVATE_SLOT(d_func(), void _q_metaDataChanged(QMap<libvlc_meta_t, QString> metaData))
    Q_PRIVATE_SLOT(d_func(), void _q_parsingFinished())
};

} // namespace CuteVlc

#endif // CUTEVLC_MEDIA_H
