/*
    Copyright (C) 2011 Harald Sitter <sitter@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MediaList.h"

#include <QtCore/QList>

#include <vlc/vlc.h>

#include "Media.h"

namespace CuteVlc {

//////////////////////////////////// Private ///////////////////////////////////

class MediaListPrivate
{
public:
    MediaListPrivate(MediaList *q) :
        q_ptr(q),
        mediaList(0)
    {
    }

    ~MediaListPrivate()
    {
        libvlc_event_manager_t *eventManager = libvlc_media_list_event_manager(mediaList);
        libvlc_event_detach(eventManager, libvlc_MediaListItemAdded, mediaListItemAdded, this);
        libvlc_event_detach(eventManager, libvlc_MediaListItemDeleted, mediaListItemDeleted, this);
        libvlc_media_list_release(mediaList);
    }

    void init()
    {
        libvlc_event_manager_t *eventManager = libvlc_media_list_event_manager(mediaList);
        libvlc_event_attach(eventManager, libvlc_MediaListItemAdded, mediaListItemAdded, this);
        libvlc_event_attach(eventManager, libvlc_MediaListItemDeleted, mediaListItemDeleted, this);
    }

#define CUTEVLC_DECLARE_EVENT_CB(x) static void x(const libvlc_event_t *event, void *vlcThat)
    CUTEVLC_DECLARE_EVENT_CB(mediaListItemAdded)
    {
        Q_ASSERT(vlcThat);
        MediaListPrivate *that = static_cast<MediaListPrivate *>(vlcThat);
        that->list.insert(event->u.media_list_item_added.index,
                          new Media(event->u.media_list_item_added.item, that->q_ptr));
    }

    CUTEVLC_DECLARE_EVENT_CB(mediaListItemDeleted)
    {
        Q_ASSERT(vlcThat);
        MediaListPrivate *that = static_cast<MediaListPrivate *>(vlcThat);
        int index = event->u.media_list_item_deleted.index;
        Media *media = that->list.at(index);
        Q_ASSERT(media);
        that->list.removeAt(index);
        media->deleteLater();
    }

#undef CUTEVLC_DECLARE_EVENT_CB

    MediaList *const q_ptr;

    QList<Media *> list;

    libvlc_media_list_t *mediaList;

private:
    V_DECLARE_PUBLIC(MediaList)
};

//////////////////////////////////// Public ////////////////////////////////////

MediaList::MediaList(libvlc_media_list_t *mediaList, QObject *parent) :
    QObject(parent),
    d_ptr(new MediaListPrivate(this))
{
    V_D(MediaList);
    d->mediaList = mediaList;
    libvlc_media_list_retain(mediaList);
    libvlc_media_list_lock(mediaList);

    int listCount = libvlc_media_list_count(mediaList);
    d->list.reserve(listCount);

    libvlc_media_t *vlcMedia;
    for (int i = 0; i < listCount; ++i) {
        vlcMedia = libvlc_media_list_item_at_index(mediaList, i);
        d->list.append(new Media(vlcMedia, this));
        libvlc_media_release(vlcMedia);
    }

    libvlc_media_list_unlock(mediaList);
    d->init();
}

MediaList::~MediaList()
{
    delete d_ptr;
}

int MediaList::size() const
{
    V_D(const MediaList);
    return d->list.size();
}

Media *MediaList::at(int i) const
{
    V_D(const MediaList);
    return d->list.at(i);
}

void MediaList::removeAt(int i)
{
    V_D(MediaList);
    libvlc_media_list_lock(d->mediaList);
    libvlc_media_list_remove_index(d->mediaList, i);
    libvlc_media_list_unlock(d->mediaList);
    Media *media = d->list.at(i);
    d->list.removeAt(i);
#warning TODO: should we really do this here?
    if (media->parent() == this) {
        media->deleteLater();
    }
}

int MediaList::indexOf(Media *media, int from) const
{
    V_D(const MediaList);
    return d->list.indexOf(media, from);
}

const QList<Media *> &MediaList::internalList() const
{
    V_D(const MediaList);
    return d->list;
}

} // namespace QtVlc
