/*
    Copyright (C) 2011 Harald Sitter <sitter@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CUTEVLC_MEDIALIST_H
#define CUTEVLC_MEDIALIST_H

#include <QtCore/QObject>

#include "Global.h"

struct libvlc_media_list_t;

namespace CuteVlc {

class Media;
class MediaListPrivate;

/**
 * \brief List container for Media instances
 *
 * This is a container for Media objects as seen in libVLC, this class however
 * contains and works with QtVlcL::Media objects rather than VLC struct instances.
 * All interfaces available in libVLC are also exposed through this class.
 *
 * Primary use of a MediaList is to represent sub items of a Media (therefore
 * each Media objected contained within a list can in turn also contain a list).
 *
 * Should you feed the MediaList with libvlc_media_t instance will those be
 * automatically be converted to QObject based Media objects.
 *
 * While this class implements a regular list it does not derive from QList, this
 * is because most manipulations also require changes to the underlying medialist
 * of libVLC. If you are aware of this and feel comfortable enough ensuring
 * consistency of data you can use the internalList() function to access the
 * list used internally by the MediaList object.
 * If you do so, you should however make sure that you are only doing trivial
 * operations such as data retrival on the list.
 *
 * \author Harald Sitter <sitter@kde.org>
 */
class CUTEVLC_EXPORT MediaList : public QObject
{
    Q_OBJECT
public:
    /**
     * Constructor.
     *
     * Constructs a new MediaList from a given libVLC media list instance.
     *
     * \param mediaList existing libVLC media list instance
     * \param parent QObject parent
     */
    explicit MediaList(libvlc_media_list_t *mediaList, QObject *parent = 0);

    /**
     * Destructor.
     *
     * Please note that the underlying libVLC medialist will be released upon
     * destruction.
     */
    ~MediaList();

    /**
     * \returns the number of Media objects in the list.
     */
    int size() const;

    /**
     * \returns the Media at index position i in the list.
     *
     * \param i must be a valid index position in the list (i.e., 0 <= i < size()).
     *
     * Just like QList's at() this function will take constat time to return.
     */
    Media *at(int i) const;

    /**
     * Removes the Media at index position i in the list.
     * If the MediaList is parent of the Media it will also be marked for deletion.
     *
     * \param i must be a valid index position in the list (i.e., 0 <= i < size()).
     */
    void removeAt(int i);

    /**
     * \returns the index position of the first occurrence of value in the list,
     * searching forward from index position from. Returns -1 if no item matched.
     *
     * \param media the Media to look for.
     * \param from at which position the search should begin.
     */
    int indexOf(Media *media, int from = 0) const;

    /**
     * The internal QList. Do not remove the constness! Using this QList to
     * manipulate data is not safe
     *
     * \returns the internally used QList
     */
    const QList<Media *> &internalList() const;

private:
    MediaListPrivate *const d_ptr;

    V_DISABLE_COPY(MediaList)
    V_DECLARE_PRIVATE(MediaList)
};

} // namespace CuteVlc

#endif // CUTEVLC_MEDIALIST_H
