/*
    Copyright (C) 2011 Harald Sitter <sitter@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MediaPlayer.h"

#include <QtCore/QDebug>
#include <QtCore/QUrl>
#include <QWidget>

#include <vlc/vlc.h>

#include "LibVLC.h"
#include "Media.h"

namespace CuteVlc {

TrackDescriptionList::TrackDescriptionList(libvlc_track_description_t *descriptor)
{
    while (descriptor) {
        append(descriptor);
        descriptor = descriptor->p_next;
    }
}

TrackDescriptionList::~TrackDescriptionList()
{
    int len = length();
    for (int i = 0; i < len; ++i) {
        release(at(i));
        removeAt(i);
    }
}

void TrackDescriptionList::release(libvlc_track_description_t *d)
{
    libvlc_track_description_release(d);
}

//////////////////////////////////// Private ///////////////////////////////////

class MediaPlayerPrivate
{
public:
    MediaPlayerPrivate(MediaPlayer *q) :
        q_ptr(q),
        player(0),
        media(0)
    {
        LibVLC::init(QList<QByteArray>());
        player = libvlc_media_player_new(libvlc);
        if (!player)
            qWarning() << "libVLC:" << LibVLC::errorMessage();
    }

    ~MediaPlayerPrivate()
    {
        libvlc_media_player_release(player);
    }

    MediaPlayer *const q_ptr;

    libvlc_media_player_t *player;

    Media *media;

private:
    V_DECLARE_PUBLIC(MediaPlayer)
};

//////////////////////////////////// Public ////////////////////////////////////

MediaPlayer::MediaPlayer(QObject *parent) :
    QObject(parent),
    d_ptr(new MediaPlayerPrivate(this))
{
}

MediaPlayer::MediaPlayer(Media *media, QObject *parent) :
    QObject(parent),
    d_ptr(new MediaPlayerPrivate(this))
{
    setMedia(media);
}

MediaPlayer::~MediaPlayer()
{
    delete d_ptr;
}

libvlc_media_player_t *MediaPlayer::vlcInstance() const
{
    V_D(const MediaPlayer);
    return d->player;
}

void MediaPlayer::setPlatformDrawable(QWidget *widget)
{
    // FIXME: need to introduce videowidget to get cocaview stuff
    V_D(MediaPlayer);
/*#if defined(Q_OS_MAC)
    libvlc_media_player_set_nsobject(m_player, m_videoWidget->cocoaView());
#el*/#if defined(Q_OS_UNIX)
    libvlc_media_player_set_xwindow(d->player, widget->winId());
#elif defined(Q_OS_WIN)
    libvlc_media_player_set_hwnd(d->player, widget->winId());
#endif
}

bool MediaPlayer::play()
{
    V_D(MediaPlayer);
    return libvlc_media_player_play(d->player);
}

bool MediaPlayer::play(QString filename)
{
    V_D(MediaPlayer);
    Media *m = new Media(filename);
    setMedia(m);
    return libvlc_media_player_play(d->player);
}

void MediaPlayer::setPause(bool pause)
{
    V_D(MediaPlayer);
    libvlc_media_player_set_pause(d->player, pause ? 1 : 0 );
}

void MediaPlayer::pause()
{
    V_D(MediaPlayer);
    libvlc_media_player_pause(d->player);
}

void MediaPlayer::stop()
{
    V_D(MediaPlayer);
    libvlc_media_player_stop(d->player);
}

libvlc_time_t MediaPlayer::length() const
{
    V_D(const MediaPlayer);
    return libvlc_media_player_get_length(d->player);
}

void MediaPlayer::setVideoCallbacks(void * (*lock)(void *, void **),
                                    void (*unlock)(void *, void *, void *const*),
                                    void (*display)(void *, void *),
                                    void *opaque)
{
    V_D(MediaPlayer);
    libvlc_video_set_callbacks(d->player, lock, unlock, display, opaque);
}

void MediaPlayer::setVideoFormat(const char *chroma,
                                 unsigned int width, unsigned int height,
                                 unsigned int bytesPerLine)
{
    V_D(MediaPlayer);
    libvlc_video_set_format(d->player, chroma, width, height, bytesPerLine);
}

Media *MediaPlayer::media() const
{
    V_D(const MediaPlayer);
    return d->media;
}

void MediaPlayer::setMedia(Media *media)
{
    V_D(MediaPlayer);
    libvlc_media_player_set_media(d->player, media->vlcInstance());
    if (d->media && d->media->parent() == this) {
        d->media->deleteLater();
    }
    d->media = media;
}

libvlc_time_t MediaPlayer::time() const
{
    V_D(const MediaPlayer);
    return libvlc_media_player_get_time(d->player);
}

void MediaPlayer::setTime(libvlc_time_t time)
{
    V_D(MediaPlayer);
    libvlc_media_player_set_time(d->player, time);
}

float MediaPlayer::position() const
{
    V_D(const MediaPlayer);
    return libvlc_media_player_get_position(d->player);
}

void MediaPlayer::setPosition(float position)
{
    V_D(MediaPlayer);
    libvlc_media_player_set_position(d->player, position);
}

TrackDescriptionList MediaPlayer::chapterDescriptionList() const
{
    V_D(const MediaPlayer);
    libvlc_track_description_t *descriptor = libvlc_video_get_spu_description(d->player);
    return TrackDescriptionList(descriptor);
}

void MediaPlayer::setChapter(int index)
{
    V_D(MediaPlayer);
    libvlc_media_player_set_chapter(d->player, index);
}

void MediaPlayer::setTitle(int index)
{
    V_D(MediaPlayer);
    libvlc_media_player_set_title(d->player, index);
}

TrackDescriptionList MediaPlayer::subtitleDescriptionList() const
{
    V_D(const MediaPlayer);
    libvlc_track_description_t *descriptor = libvlc_video_get_spu_description(d->player);
    return TrackDescriptionList(descriptor);
}

bool MediaPlayer::setSubtitle(int index)
{
    V_D(MediaPlayer);
    return libvlc_video_set_spu(d->player, index);
}

bool MediaPlayer::setSubtitleFile(const QString &file)
{
    V_D(MediaPlayer);
    return libvlc_video_set_subtitle_file(d->player, file.toUtf8().constData());
}

bool MediaPlayer::setSubtitleFile(const QUrl &file)
{
    return setSubtitleFile(file.toString());
}

TrackDescriptionList MediaPlayer::audioTrackDescriptionList() const
{
    V_D(const MediaPlayer);
    libvlc_track_description_t *descriptor = libvlc_audio_get_track_description(d->player);
    return TrackDescriptionList(descriptor);
}

int MediaPlayer::audioTrack() const
{
    V_D(const MediaPlayer);
    return libvlc_audio_get_track(d->player);
}

bool MediaPlayer::setAudioTrack(int index)
{
    V_D(const MediaPlayer);
    return libvlc_audio_set_track(d->player, index);
}

} // namespace QtVlc
