/*
    Copyright (C) 2011 Harald Sitter <sitter@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CUTEVLC_MEDIAPLAYER_H
#define CUTEVLC_MEDIAPLAYER_H

#include <QtCore/QObject>

extern "C"
{
#include <vlc/libvlc.h>
#include <vlc/libvlc_structures.h>
}

#include "Global.h"

class QUrl;

struct libvlc_media_player_t;
struct libvlc_track_description_t;

namespace CuteVlc {

class Media;
class MediaPlayerPrivate;

#warning move elsewhere
class CUTEVLC_EXPORT TrackDescriptionList : public QList<libvlc_track_description_t *>
{
public:
    TrackDescriptionList(libvlc_track_description_t *descriptor);
    ~TrackDescriptionList();

protected:
    virtual void release(libvlc_track_description_t *d);
};

/**
 * \brief A tiny Qt wrapper around LibVLC's media_player
 *
 * This class is neither thread-safe nor reentrant.
 *
 * \author Harald Siter <sitter@kde.org>
 */
class CUTEVLC_EXPORT MediaPlayer : public QObject
{
    Q_OBJECT
public:
    /**
     * Constucts an empty MediaPlayer.
     *
     * \param parent the Qt parent for the MediaPlayer
     */
    explicit MediaPlayer(QObject *parent = 0);

    /**
     * Constructs a MediaPlayer and sets a Media for it.
     *
     * \param media the media to set
     * \param parent the Qt parent for the MediaPlayer
     */
    explicit MediaPlayer(Media *media, QObject *parent = 0);

    /**
     * Destructor
     *
     * Please note that the underlying libvlc media_player will be released upon destruction.
     */
    virtual ~MediaPlayer();

    /**
     * \returns the actual libvlc media player instance held by the object.
     */
    libvlc_media_player_t *vlcInstance() const;

    /**
     * Set a platform drawable for video.
     * TODO
     * \param widget needs to be a video widget suited for the platform
     */
    void setPlatformDrawable(QWidget *widget);

    /**
     * Start playing.
     *
     * \returns whether playback was started
     */
    Q_INVOKABLE bool play();

    /**
     * Start playing.
     *
     * \returns whether playback was started
     */
    Q_INVOKABLE bool play(QString filename);

    /**
     * Pause or resume (no effect if there is no media)
     *
     * \param pause whether to pause or to resume
     */
    void setPause(bool pause);

    /**
     * Toggle pause (if at all possible).
     */
    void pause();

    /**
     * Stops playback (if at all possible).
     */
    void stop();

    /**
     * Get the current movie length (in ms).
     *
     * \returns the movie length (in ms), or -1 if there is no media.
     */
    // FIXME libvlc_time_t include waaaah!!!!
    libvlc_time_t length() const;

    /**
     * Set callbacks and private data to render decoded video to a custom area
     * in memory. Use setVideoFormat() to configure the decoded format.
     *
     * Whenever a new video frame needs to be decoded, the lock callback is
     * invoked. Depending on the video chroma, one or three pixel planes of
     * adequate dimensions must be returned via the second parameter. Those
     * planes must be aligned on 32-bytes boundaries.
     *
     * When the video frame is decoded, the unlock callback is invoked. The
     * second parameter to the callback corresponds is the return value of the
     * lock callback. The third parameter conveys the pixel planes for convenience.
     *
     * When the video frame needs to be shown, as determined by the media playback
     * clock, the display callback is invoked. The second parameter also conveys
     * the return value from the lock callback.
     *
     * \param lock callback to lock video memory (must not be 0)
     * \param unlock callback to unlock video memory (or 0 if not needed)
     * \param display callback to display video (or 0 if not needed)
     * \param opaque private pointer for the three callbacks (as first parameter)
     *
     * \see setVideoFormat
     */
    void setVideoCallbacks(void *(*lock) (void *opaque, void **plane),
                           void (*unlock) (void *opaque, void *picture, void *const *plane),
                           void (*display) (void *opaque, void *picture),
                           void *opaque);

    /**
     * Set decoded video chroma and dimensions. This only works in combination with
     * setVideoCallbacks().
     *
     * \param chroma a four-characters string identifying the chroma
     *               (e.g. "RV32" or "I420")
     * \param width pixel width
     * \param height pixel height
     * \param bytesPerLine amount of bytes per line (also known as line pitch)
     *
     * \see setVideoCallbacks
     */
    void setVideoFormat(const char *chroma,
                        unsigned int width, unsigned int height,
                        unsigned int bytesPerLine);

    /**
     * \returns currently set Media or 0
     */
    Media *media() const;

    /**
     * Set the Media that will be used by the MediaPlayer. Any previous Media will
     * be deleted iff its QObject parent is the MediaPlayer itself.
     * Consequently if you want to stop caring about the Media you just need to set
     * its parent to the MediaPlayer after a call to setMedia and any subsequent
     * Media change or destruction of the player will also destroy the Media object.
     *
     * \param media the Media - must not be deleted!
     */
    void setMedia(Media *media);

    /**
     * Get the current movie time (in ms).
     *
     * \return the movie time (in ms), or -1 if there is no media.
     */
    libvlc_time_t time() const;

    /**
     * Set the movie time (in ms). This has no effect if no media is being played.
     * Not all formats and protocols support this.
     *
     * \param time the movie time (in ms).
     */
    void setTime(libvlc_time_t time);

    /**
     * Get movie position.
     *
     * \return movie position, or -1. in case of error
     */
    float position() const;

    /**
     * Set movie position. This has no effect if playback is not enabled.
     * This might not do anything depending on the underlying input format and protocol.
     *
     * \param position the position
     */
    void setPosition(float position);

    TrackDescriptionList chapterDescriptionList() const;
    void setChapter(int index);

    void setTitle(int index);

    TrackDescriptionList subtitleDescriptionList() const;
    bool setSubtitle(int index);
    bool setSubtitleFile(const QString &file);
    bool setSubtitleFile(const QUrl &file);

    /**
     * Get the descriptions of available audio tracks.
     *
     * \return list with description of available audio tracks
     */
    TrackDescriptionList audioTrackDescriptionList() const;

    /**
     * Get current audio track.
     *
     * \return the audio track index, or -1 if none.
     */
    int audioTrack() const;

    /**
     * Set current audio track.
     *
     * \param index the track index
     * \return whether action was successful
     */
    bool setAudioTrack(int index);

private:
    MediaPlayerPrivate *const d_ptr;

    V_DISABLE_COPY(MediaPlayer)
    V_DECLARE_PRIVATE(MediaPlayer)
};

} // namespace CuteVlc

#endif // CUTEVLC_MEDIAPLAYER_H
